# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import plugin_pb2 as plugin__pb2


class PluginServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Register = channel.unary_unary(
                '/plugin.PluginService/Register',
                request_serializer=plugin__pb2.RegisterRequest.SerializeToString,
                response_deserializer=plugin__pb2.RegisterResponse.FromString,
                )
        self.Call = channel.unary_unary(
                '/plugin.PluginService/Call',
                request_serializer=plugin__pb2.EndpointRequestMessage.SerializeToString,
                response_deserializer=plugin__pb2.EndpointResponseMessage.FromString,
                )
        self.LoggingStream = channel.unary_stream(
                '/plugin.PluginService/LoggingStream',
                request_serializer=plugin__pb2.LoggingArgs.SerializeToString,
                response_deserializer=plugin__pb2.LogMessage.FromString,
                )


class PluginServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def Register(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Call(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def LoggingStream(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_PluginServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Register': grpc.unary_unary_rpc_method_handler(
                    servicer.Register,
                    request_deserializer=plugin__pb2.RegisterRequest.FromString,
                    response_serializer=plugin__pb2.RegisterResponse.SerializeToString,
            ),
            'Call': grpc.unary_unary_rpc_method_handler(
                    servicer.Call,
                    request_deserializer=plugin__pb2.EndpointRequestMessage.FromString,
                    response_serializer=plugin__pb2.EndpointResponseMessage.SerializeToString,
            ),
            'LoggingStream': grpc.unary_stream_rpc_method_handler(
                    servicer.LoggingStream,
                    request_deserializer=plugin__pb2.LoggingArgs.FromString,
                    response_serializer=plugin__pb2.LogMessage.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'plugin.PluginService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class PluginService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def Register(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/plugin.PluginService/Register',
            plugin__pb2.RegisterRequest.SerializeToString,
            plugin__pb2.RegisterResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Call(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/plugin.PluginService/Call',
            plugin__pb2.EndpointRequestMessage.SerializeToString,
            plugin__pb2.EndpointResponseMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def LoggingStream(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/plugin.PluginService/LoggingStream',
            plugin__pb2.LoggingArgs.SerializeToString,
            plugin__pb2.LogMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import json
import typing
from enum import Enum
from ..host.plugin_pb2 import LogLevel as apiLogLevel
from ..host.plugin_pb2 import StringList, EndpointRequest as apiEndpointRequest, EndpointResponse as apiEndpointResponse
from ..host.helpers.debug_sender import DebugSender

class AuthGroup(Enum):
    GroupGuest = "guest"
    GroupUser = "user"
    GroupOperator = "operator"
    GroupAdmin = "admin"

# EndpointAction is an enum that represents the actions that the plugin can perform
class EndpointAction(Enum):
    ActionRead = "read"
    ActionWrite = "write"
    ActionDelete = "delete"
    ActionCreate = "create"

class LoggingLevel(Enum):
    LevelDebug = 0
    LevelInfo = 1
    LevelWarning = 2
    LevelError = 3
    LevelFatal = 4

    def to_api_level(self):
        if self == LoggingLevel.LevelDebug:
            return apiLogLevel.Debug
        elif self == LoggingLevel.LevelInfo:
            return apiLogLevel.INFO
        elif self == LoggingLevel.LevelWarning:
            return apiLogLevel.WARNING
        elif self == LoggingLevel.LevelError:
            return apiLogLevel.ERROR
        elif self == LoggingLevel.LevelFatal:
            return apiLogLevel.FATAL
        else:
            return apiLogLevel.ERROR

class LogMessage:
    def __init__(self, level, message, fields):
        self.level = level
        self.message = message
        self.fields = fields

# EndpointRequest is a class that represents the request that is sent to the plugin from the DTAC Agent
# it uses composition and redirection to expose the attributes of the grpc class without exposing the class
# itself directly to SDK users
class EndpointRequest:
    #
    # metadata: typing.Mapping[str, str] | None
    # headers: typing.Mapping[str, StringList] | None
    # parameters: typing.Mapping[str, StringList] | None
    # body: bytes | None

    def __init__(self, *args, **kwargs):
        # Initialize the composed object without setting attributes
        self._api_endpoint_request = apiEndpointRequest()

        # Use __setattr__ to set attributes
        for key, value in kwargs.items():
            self.__setattr__(key, value)

    def __getattr__(self, item):
        value = getattr(self._api_endpoint_request, item)
        if item == 'headers' or item == 'parameters':
            return {k: list(v.values) for k, v in value.items()}
        return value

    def __setattr__(self, key, value):
        if key == '_api_endpoint_request':
            super().__setattr__(key, value)
        elif key == 'headers' or key == 'parameters':
            # Convert list to StringList
            string_list_value = {k: StringList(values=v) for k, v in value.items()}
            setattr(self._api_endpoint_request, key, string_list_value)
        else:
            setattr(self._api_endpoint_request, key, value)

    def to_api_request(self):
        return self._api_endpoint_request

    @staticmethod
    def from_api_request(api_request: apiEndpointRequest):
        # Create an SDK object from a gRPC object
        request = EndpointRequest()
        request._api_endpoint_request = api_request
        return request

# EndpointResponse is a class that represents the request that is sent to the plugin from the DTAC Agent
# it uses composition and redirection to expose the attributes of the grpc class without exposing the class
# itself directly to SDK users
class EndpointResponse:
    #
    # metadata: typing.Mapping[str, str] | None
    # headers: typing.Mapping[str, StringList] | None
    # parameters: typing.Mapping[str, StringList] | None
    # body: bytes | None

    def __init__(self, headers: typing.Mapping[str, typing.List[str]], parameters: typing.Mapping[str, typing.List[str]], *args, **kwargs):
        # Initialize the composed object without setting attributes
        self._api_endpoint_response = apiEndpointResponse()

        # Use __setattr__ to set attributes
        for key, value in kwargs.items():
            self.__setattr__(key, value)

    def __getattr__(self, item):
        value = getattr(self._api_endpoint_response, item)
        if item == 'headers' or item == 'parameters':
            return {k: list(v.values) for k, v in value.items()}
        return value

    def __setattr__(self, key, value):
        if key == '_api_endpoint_response':
            super().__setattr__(key, value)
        elif key == 'headers':
            # Convert list to StringList
            headers = {k: StringList(values=v) for k, v in value.items()}
            for key, value in headers.items():
                self._api_endpoint_response.headers[key].values.extend(value)
        elif key == 'parameters':
            # Convert list to StringList
            parameters = {k: StringList(values=v) for k, v in value.items()}
            for key, value in parameters.items():
                self._api_endpoint_response.parameters[key].values.extend(value)
        elif key == 'value':
            # Check if the value is already a byte array
            if not isinstance(value, bytes):
                try:
                    # Serialize to JSON and then to bytes
                    serialized_value = json.dumps(value).encode('utf-8')
                except TypeError:
                    raise ValueError("Provided value is not serializable")
                self._api_endpoint_response.value = serialized_value
            else:
                self._api_endpoint_response.value = value
        else:
            setattr(self._api_endpoint_response, key, value)

    def to_api_response(self):
        return self._api_endpoint_response

    @staticmethod
    def from_api_response(api_response: apiEndpointResponse):
        # Create an SDK object from a gRPC object
        response = EndpointResponse()
        response._api_endpoint_response = api_response
        return response

class PluginEndpoint:
    def __init__(self,
                 function: typing.Callable,
                 path: str,
                 action: str,
                 description: str,
                 secure: bool = True,
                 auth_group: str | None = None,
                 expected_metadata_schema: str | None = None,
                 expected_headers_schema: str | None = None,
                 expected_parameters_schema: str | None = None,
                 expected_body_schema: str | None = None,
                 expected_output_schema: str | None = None
                 ):
        self.function = function
        self.function_name = function.__name__
        self.path = path
        self.action = action
        self.description = description
        self.secure = secure
        self.auth_group = auth_group
        self.expected_metadata_schema = expected_metadata_schema
        self.expected_headers_schema = expected_headers_schema
        self.expected_parameters_schema = expected_parameters_schema
        self.expected_body_schema = expected_body_schema
        self.expected_output_schema = expected_output_schema

    def to_dict(self):
        return {
            "function_name": self.function_name,
            "path": self.path,
            "action": self.action,
            "description": self.description,
            "secure": self.secure,
            "auth_group": self.auth_group,
            "expected_metadata_schema": self.expected_metadata_schema,
            "expected_headers_schema": self.expected_headers_schema,
            "expected_parameters_schema": self.expected_parameters_schema,
            "expected_body_schema": self.expected_body_schema,
            "expected_output_schema": self.expected_output_schema
        }

    def to_json(self):
        return json.dumps(self, default=lambda o: {k: v for k, v in o.__dict__.items() if v is not None and not callable(v)}, indent=4)



# class InputArgs:
#     def __init__(self, headers=None, params=None, body=None):
#         if headers is None:
#             headers = {}
#         if params is None:
#             params = {}
#         self.headers = headers
#         self.params = params
#         self.body = body
#
#     def to_dict(self):
#         return {
#             "headers": self.headers,
#             "params": self.params,
#             "body": self.body.decode('utf-8') if self.body else None
#         }
#
#     def to_json(self):
#         return json.dumps(self.to_dict())
#
#     @classmethod
#     def from_json(cls, json_str):
#         data = json.loads(json_str)
#         body = data.get("body")
#         if body is not None:
#             body = body.encode('utf-8')
#         return cls(
#             headers=data.get("headers"),
#             params=data.get("params"),
#             body=body
#         )
#
#     @classmethod
#     def from_dict(cls, data):
#         body = data.get("body")
#         if body is not None:
#             body = body.encode('utf-8')
#         return cls(
#             headers=data.get("headers"),
#             params=data.get("params"),
#             body=body
#         )
#
#     @classmethod
#     def from_grpc(cls, grpc_obj: ApiInputArgs):
#         headers = {}
#         # for key, value in grpc_obj.headers.items():
#         #     headers[key] = value
#         #
#         params = {}
#         # for key, value in grpc_obj.params.items():
#         #     params[key] = value
#
#         return cls(
#             headers=headers,
#             params=params,
#             body=grpc_obj.body
#         )
#
# class ReturnVal:
#     def __init__(self, headers=None, params=None, value=None):
#         if headers is None:
#             headers = {}
#         if params is None:
#             params = {}
#         self.headers = headers
#         self.params = params
#         self.value = value
#
#     def value_to_json(self):
#         # Convert the value attribute to a JSON string
#         try:
#             return json.dumps(self.value)
#         except TypeError as e:
#             # Handle the error if value is not serializable
#             return str(e)
#
#     def to_dict(self):
#         return {
#             "headers": self.headers,
#             "params": self.params,
#             "value": self.value
#         }
#
#     def to_json(self):
#         return json.dumps(self.to_dict())
#
#     def to_grpc(self):
#         headers = {}
#         for key, values in self.headers.items():
#             headers[key] = StringList(values=values)
#
#         params = {}
#         for key, values in self.params.items():
#             params[key] = StringList(values=values)
#
#         return ApiReturnVal(
#             headers=headers,
#             params=params,
#             value=self.value
#         )


# default_host.py
import os
import queue
import sys
import json
import grpc
import traceback
import dtac_tools.host.plugin_pb2_grpc

from grpc import ssl_server_credentials
from urllib.parse import quote
from concurrent import futures
from .helpers.debug_sender import DebugSender
from .helpers.encryptor import RpcEncryptor
from .plugin_host import PluginHost
from .helpers.network import get_unused_tcp_port
from ..plugins.types import EndpointRequest, EndpointResponse, AuthGroup
from .plugin_pb2 import EndpointRequestMessage, EndpointResponseMessage, RegisterRequest, RegisterResponse, LogField, LogMessage, PluginEndpoint


class DefaultPluginHost(PluginHost):
    def __init__(self, plugin, debug=False, debug_port=5678):
        self.plugin = plugin
        self.rpc_proto = "grpc"
        self.proto = "tcp"
        self.ip = "127.0.0.1"
        self.interface_version = 'plug_api_1.0'
        self.port = None
        self.route_map = {}
        self.encryptor = RpcEncryptor.new_encryptor()
        self.log_channel = None
        self.debug_sender = None
        self.default_secure = True
        if debug:
            self.debug_sender = DebugSender(debug_port)
            self.debug(f"PluginHost debugging enabled on port {debug_port}")
            self.debug(f"Plugin Name: {self.plugin.name()}")

    def debug(self, message):
        if hasattr(self, "debug_sender") and self.debug_sender is not None:
            self.debug_sender.write(message + "\n")

    # Register is the gRPC handler for the plugin. It will be called by the DTAC Agent when the plugin is loaded and will
    # be used to register the plugin with the DTAC Agent and to set up the endpoints that the plugin will handle.
    def Register(self, request: RegisterRequest, context: grpc.ServicerContext) -> RegisterResponse:
        try :
            params = {}
            if request.config is not None:
                self.debug(f"config: {request.config}")
                params["config"] = request.config

            if request.default_secure is not None:
                self.debug(f"default_secure: {request.default_secure}")
                params["default_secure"] = request.default_secure
                self.plugin.set_default_secure(request.default_secure)

            response = self.plugin.register(params)

            # build the route map
            for endpoint in response:
                self.route_map[endpoint.path] = endpoint

            return_eps = []
            for endpoint in response:
                # Ensure auth_group is set. If it is not set then set it to the most secure auth group
                if endpoint.auth_group is None:
                    endpoint.auth_group = AuthGroup.GroupAdmin

                api_ep = PluginEndpoint(
                    path = endpoint.path,
                    action = endpoint.action,
                    secure = endpoint.secure,
                    auth_group = endpoint.auth_group.value,
                    expected_metadata_schema = endpoint.expected_metadata_schema,
                    expected_headers_schema = endpoint.expected_headers_schema,
                    expected_parameters_schema = endpoint.expected_parameters_schema,
                    expected_body_schema = endpoint.expected_body_schema,
                    expected_output_schema = endpoint.expected_output_schema,
                )
                return_eps.append(api_ep)

            self.debug(f"response: {response}")
            return_val = RegisterResponse(
                endpoints = return_eps,
            )
            return return_val
        except Exception as ex:
            self.debug(f"Exception: {ex}")
            self.debug(traceback.format_exc())
            return RegisterResponse()


    # Call is the gRPC handler for the plugin. It will be called by the DTAC Agent when a request is made to the plugin
    # and will be used to route the call to the correct endpoint handler and return the response to the DTAC Agent
    def Call(self, request: EndpointRequestMessage, context: grpc.ServicerContext) -> EndpointResponseMessage:
        try:
            method = request.method
            request_args = EndpointRequest.from_api_request(request.request)
            self.debug(f"method: {method}")
            self.debug(f"request_args: {request_args}")
            function = self.route_map[method].function
            ret = function(request_args)
            self.debug(f"output: {ret}")

            return EndpointResponseMessage(
                id = 1234, # This isn't used by Python so just populate it with a dummy value
                response = ret.to_api_response(),
                error = "", # This is a vestage of the old API and should likely be removed as gRPC can return errors directly
            )

        except Exception as ex:
            self.debug(f"Exception: {ex}")
            self.debug(traceback.format_exc())
            context.set_code(grpc.StatusCode.INTERNAL)
            context.set_details('Internal error: {}'.format(ex))
            return EndpointResponseMessage()


    # LoggingStream is the gRPC handler for the logging stream. It will return a stream of log messages to the DTAC Agent
    def LoggingStream(self, request, context):
        try:
            if self.plugin.log_channel is None:
                self.plugin.log_channel = queue.Queue(maxsize=4096)
        except Exception as ex:
            self.debug(f"Exception: {ex}")
            self.debug(traceback.format_exc())
            context.set_code(grpc.StatusCode.INTERNAL)
            context.set_details('Internal error: {}'.format(ex))
            return LogMessage()

        while True:
            try:
                msg = self.plugin.log_channel.get()
                yield msg
            except Exception as e:
                self.debug(f"Logging Exception: {e}")
                context.set_code(grpc.StatusCode.INTERNAL)
                context.set_details('Logging error: {}'.format(e))
                break

    # serve is the main entry point for the plugin. It will start the gRPC server and listen for connections from the
    # DTAC Agent
    def serve(self):

        env_cookie = os.getenv("DTAC_PLUGINS")
        if env_cookie is None:
            print('============================ WARNING ============================')
            print('This is a DTAC plugin and is not designed to be executed directly')
            print('Please use the DTAC agent to load this plugin')
            print('==================================================================')
            sys.exit(-1)

        self.port = get_unused_tcp_port()

        # Check for certificate and key files passed via ENV variables
        cert = os.getenv("DTAC_TLS_CERT")
        key = os.getenv("DTAC_TLS_KEY")

        tls = True if cert is not "" and key is not "" else False
        options = [
            f'enc={quote(self.encryptor.key_string())}',
            f'tls={tls}',
        ]



        print(f"CONNECT{{{{{self.plugin.name()}:{self.plugin.root_path()}:{self.rpc_proto}:{self.proto}:{self.ip}:{self.port}:{self.interface_version}:[{','.join(options)}]}}}}")
        sys.stdout.flush()



        # Create a gRPC server
        server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))

        # Check if both certificate and key are provided
        if tls:
            # Convert certificate and key strings into bytes
            certificate_chain = cert.encode('utf-8')
            private_key = key.encode('utf-8')

            # Create server SSL credentials
            server_credentials = ssl_server_credentials(
                [(private_key, certificate_chain)]
            )

            # Add secure port using credentials
            server.add_secure_port(f"[::]:{self.port}", server_credentials)
        else:
            # Add insecure port if no TLS credentials are provided
            server.add_insecure_port(f"[::]:{self.port}")

        dtac_tools.host.plugin_pb2_grpc.add_PluginServiceServicer_to_server(self, server)
        server.start()
        server.wait_for_termination()

    # get_port returns the port that the plugin is listening for connection on
    def get_port(self):
        return self.port


import json
import queue
import traceback
import typing

from pydantic import BaseModel
from typing import TypeVar, Type
from ..host.helpers.debug_sender import DebugSender
from .types import LoggingLevel, PluginEndpoint, EndpointAction, AuthGroup
from ..host.plugin_pb2 import LogField as apiLogField, LogLevel as apiLogLevel, LogMessage as apiLogMessage

# Base Config class, this is used to help deserialize configuration options for plugins
class BaseConfig(BaseModel):
    pass


class PluginShared:
    @staticmethod
    def name() -> str:
        return "UnnamedPlugin"
    
    @staticmethod
    def root_path() -> str:
        return ""
    
    @staticmethod
    def serialize(v: object) -> bytes:
        return json.dumps(v).encode('utf-8')

class PluginBase(PluginShared):

    T = TypeVar("T", bound=BaseConfig)
    log_channel: queue.Queue = None
    default_secure: bool = True

    # set_default_secure is a helper function that allows plugins to set the default secure value for all endpoints
    def set_default_secure(self, default_secure: bool):
        self.default_secure = default_secure

    # enable_debugging enables debugging for the plugin. This will open a port on the local machine and will allow the
    # plugin to send debug messages to the local udp port. This is not intended on being long-lived and is only meant
    # to be used for debugging purposes during the early development of this sdk
    def enable_debugging(self):
        self.debug_sender = DebugSender(5678)
        self.debug(f"Plugin debugging enabled on port {5678}")
        self.debug(f"Plugin Name: {self.name()}")

    # debug prints out a debug message over the debug_sender if it is enabled.
    def debug(self, message):
        if hasattr(self, "debug_sender") and self.debug_sender is not None:
            self.debug_sender.write(message + "\n")

    # load_config is a helper class that will deserialize the configuration json into the provided config_class
    def load_config(self, config_json: str, config_class: Type[T]) -> T:
        return config_class.parse_raw(config_json)

    # register is a function that must be in all plugins with this signature. It is used to setup the handlers that the
    # plugin will support. At this level in the code it is just here to provide a default implementation that will
    # raise a NotImplementedError if it is not overridden by the plugin.
    def register(self, args):
        raise NotImplementedError("this method must be implemented")

    # log is a function to allow plugins to use the central logging facilities of the DTAC Agent. It takes a log_level
    # which is a LoggingLevel enum value, a message string, and a fields dictionary which is a dictionary of key/value
    def log(self, log_level: LoggingLevel, message: str, fields: dict):
        try:
            if self.log_channel is None:
                self.log_channel = queue.Queue(maxsize=4096)

            msg_fields = []
            for k, v in fields.items():
                msg_fields.append(apiLogField(key=k, value=v))

            log_message = apiLogMessage(level=log_level.to_api_level(), message=message, fields=msg_fields)
            self.log_channel.put(log_message)
        except Exception as ex:
            self.debug(f"Exception: {ex}")
            self.debug(traceback.format_exc())

    # name is a function that gets the name of the class which is used as the name of the plugin. --TOOD: check to see
    # if this is still needed as it shouldn't be since the default is to use the name of the plugin file minus the
    # .plugin extension as seen by the DTAC Agent.
    def name(self) -> str:
        return self.__class__.__name__

    def new_read_endpoint(self,
                          function: typing.Callable,
                          path: str,
                          description: str,
                          override_default_secure: bool = False,
                          secure: bool = True,
                          auth_group: AuthGroup | None = None,
                          expected_headers_schema: str | None = None,
                          expected_parameters_schema: str | None = None,
                          expected_body_schema: str | None = None,
                          expected_output_schema: str | None = None) -> PluginEndpoint:
        return self.new_endpoint(function=function, path=path,
                            action=EndpointAction.ActionRead.value,
                            description=description,
                                 override_default_secure=override_default_secure,
                                 secure=secure,
                                 auth_group=auth_group,
                                 expected_headers_schema=expected_headers_schema,
                                 expected_parameters_schema=expected_parameters_schema,
                                 expected_body_schema=expected_body_schema,
                                 expected_output_schema=expected_output_schema)

    def new_write_endpoint(self,
                           function: typing.Callable,
                           path: str,
                           description: str,
                           override_default_secure: bool = False,
                           secure: bool = True,
                           auth_group: AuthGroup | None = None,
                           expected_headers_schema: str | None = None,
                           expected_parameters_schema: str | None = None,
                           expected_body_schema: str | None = None,
                           expected_output_schema: str | None = None) -> PluginEndpoint:
        return self.new_endpoint(function=function, path=path,
                                 action=EndpointAction.ActionWrite.value,
                                 description=description,
                                 override_default_secure=override_default_secure,
                                 secure=secure,
                                 auth_group=auth_group,
                                 expected_headers_schema=expected_headers_schema,
                                 expected_parameters_schema=expected_parameters_schema,
                                 expected_body_schema=expected_body_schema,
                                 expected_output_schema=expected_output_schema)

    def new_create_endpoint(self,
                            function: typing.Callable,
                            path: str,
                            description: str,
                            override_default_secure: bool = False,
                            secure: bool = True,
                            auth_group: AuthGroup | None = None,
                            expected_headers_schema: str | None = None,
                            expected_parameters_schema: str | None = None,
                            expected_body_schema: str | None = None,
                            expected_output_schema: str | None = None) -> PluginEndpoint:
        return self.new_endpoint(function=function, path=path,
                                 action=EndpointAction.ActionCreate.value,
                                 description=description,
                                 override_default_secure=override_default_secure,
                                 secure=secure,
                                 auth_group=auth_group,
                                 expected_headers_schema=expected_headers_schema,
                                 expected_parameters_schema=expected_parameters_schema,
                                 expected_body_schema=expected_body_schema,
                                 expected_output_schema=expected_output_schema)

    def new_delete_endpoint(self,
                            function: typing.Callable,
                            path: str,
                            description: str,
                            override_default_secure: bool = False,
                            secure: bool = True,
                            auth_group: AuthGroup | None = None,
                            expected_headers_schema: str | None = None,
                            expected_parameters_schema: str | None = None,
                            expected_body_schema: str | None = None,
                            expected_output_schema: str | None = None) -> PluginEndpoint:
        return self.new_endpoint(function=function, path=path,
                                 action=EndpointAction.ActionDelete.value,
                                 description=description,
                                 override_default_secure=override_default_secure,
                                 secure=secure,
                                 auth_group=auth_group,
                                 expected_headers_schema=expected_headers_schema,
                                 expected_parameters_schema=expected_parameters_schema,
                                 expected_body_schema=expected_body_schema,
                                 expected_output_schema=expected_output_schema)

    def new_endpoint(self,
                     function: typing.Callable,
                     path: str,
                     action: str,
                     description: str,
                     override_default_secure: bool = False,
                     secure: bool = True,
                     auth_group: AuthGroup | None = None,
                     expected_headers_schema: str | None = None,
                     expected_parameters_schema: str | None = None,
                     expected_body_schema: str | None = None,
                     expected_output_schema: str | None = None) -> PluginEndpoint:
        make_secure = self.default_secure
        if override_default_secure:
            make_secure = secure

        return PluginEndpoint(
            function=function,
            path=path,
            action=action,
            description=description,
            secure=make_secure,
            auth_group=auth_group,
            expected_headers_schema=expected_headers_schema,
            expected_parameters_schema=expected_parameters_schema,
            expected_body_schema=expected_body_schema,
            expected_output_schema=expected_output_schema
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: plugin.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0cplugin.proto\x12\x06plugin\"R\n\x16\x45ndpointRequestMessage\x12\x0e\n\x06method\x18\x01 \x01(\t\x12(\n\x07request\x18\x02 \x01(\x0b\x32\x17.plugin.EndpointRequest\"`\n\x17\x45ndpointResponseMessage\x12\n\n\x02id\x18\x01 \x01(\x05\x12*\n\x08response\x18\x02 \x01(\x0b\x32\x18.plugin.EndpointResponse\x12\r\n\x05\x65rror\x18\x03 \x01(\t\"\x88\x03\n\x0f\x45ndpointRequest\x12\x37\n\x08metadata\x18\x01 \x03(\x0b\x32%.plugin.EndpointRequest.MetadataEntry\x12\x35\n\x07headers\x18\x02 \x03(\x0b\x32$.plugin.EndpointRequest.HeadersEntry\x12;\n\nparameters\x18\x03 \x03(\x0b\x32\'.plugin.EndpointRequest.ParametersEntry\x12\x0c\n\x04\x62ody\x18\x04 \x01(\x0c\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x42\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.plugin.StringList:\x02\x38\x01\x1a\x45\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.plugin.StringList:\x02\x38\x01\"\x8d\x03\n\x10\x45ndpointResponse\x12\x38\n\x08metadata\x18\x01 \x03(\x0b\x32&.plugin.EndpointResponse.MetadataEntry\x12\x36\n\x07headers\x18\x02 \x03(\x0b\x32%.plugin.EndpointResponse.HeadersEntry\x12<\n\nparameters\x18\x03 \x03(\x0b\x32(.plugin.EndpointResponse.ParametersEntry\x12\r\n\x05value\x18\x04 \x01(\x0c\x1a/\n\rMetadataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x42\n\x0cHeadersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.plugin.StringList:\x02\x38\x01\x1a\x45\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.plugin.StringList:\x02\x38\x01\"\x1c\n\nStringList\x12\x0e\n\x06values\x18\x01 \x03(\t\"9\n\x0fRegisterRequest\x12\x0e\n\x06\x63onfig\x18\x01 \x01(\t\x12\x16\n\x0e\x64\x65\x66\x61ult_secure\x18\x02 \x01(\x08\"=\n\x10RegisterResponse\x12)\n\tendpoints\x18\x01 \x03(\x0b\x32\x16.plugin.PluginEndpoint\"\x8c\x02\n\x0ePluginEndpoint\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0e\n\x06\x61\x63tion\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x0e\n\x06secure\x18\x04 \x01(\x08\x12\x12\n\nauth_group\x18\x05 \x01(\t\x12 \n\x18\x65xpected_metadata_schema\x18\x06 \x01(\t\x12\x1f\n\x17\x65xpected_headers_schema\x18\x07 \x01(\t\x12\"\n\x1a\x65xpected_parameters_schema\x18\x08 \x01(\t\x12\x1c\n\x14\x65xpected_body_schema\x18\t \x01(\t\x12\x1e\n\x16\x65xpected_output_schema\x18\n \x01(\t\"\r\n\x0bLoggingArgs\"&\n\x08LogField\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"`\n\nLogMessage\x12\x1f\n\x05level\x18\x01 \x01(\x0e\x32\x10.plugin.LogLevel\x12\x0f\n\x07message\x18\x02 \x01(\t\x12 \n\x06\x66ields\x18\x03 \x03(\x0b\x32\x10.plugin.LogField*B\n\x08LogLevel\x12\t\n\x05\x44\x45\x42UG\x10\x00\x12\x08\n\x04INFO\x10\x01\x12\x0b\n\x07WARNING\x10\x02\x12\t\n\x05\x45RROR\x10\x03\x12\t\n\x05\x46\x41TAL\x10\x04\x32\xd3\x01\n\rPluginService\x12=\n\x08Register\x12\x17.plugin.RegisterRequest\x1a\x18.plugin.RegisterResponse\x12G\n\x04\x43\x61ll\x12\x1e.plugin.EndpointRequestMessage\x1a\x1f.plugin.EndpointResponseMessage\x12:\n\rLoggingStream\x12\x13.plugin.LoggingArgs\x1a\x12.plugin.LogMessage0\x01\x42KZIgithub.com/intel-innersource/frameworks.automation.dtac.agent/api/grpc/gob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'plugin_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'ZIgithub.com/intel-innersource/frameworks.automation.dtac.agent/api/grpc/go'
  _ENDPOINTREQUEST_METADATAENTRY._options = None
  _ENDPOINTREQUEST_METADATAENTRY._serialized_options = b'8\001'
  _ENDPOINTREQUEST_HEADERSENTRY._options = None
  _ENDPOINTREQUEST_HEADERSENTRY._serialized_options = b'8\001'
  _ENDPOINTREQUEST_PARAMETERSENTRY._options = None
  _ENDPOINTREQUEST_PARAMETERSENTRY._serialized_options = b'8\001'
  _ENDPOINTRESPONSE_METADATAENTRY._options = None
  _ENDPOINTRESPONSE_METADATAENTRY._serialized_options = b'8\001'
  _ENDPOINTRESPONSE_HEADERSENTRY._options = None
  _ENDPOINTRESPONSE_HEADERSENTRY._serialized_options = b'8\001'
  _ENDPOINTRESPONSE_PARAMETERSENTRY._options = None
  _ENDPOINTRESPONSE_PARAMETERSENTRY._serialized_options = b'8\001'
  _globals['_LOGLEVEL']._serialized_start=1577
  _globals['_LOGLEVEL']._serialized_end=1643
  _globals['_ENDPOINTREQUESTMESSAGE']._serialized_start=24
  _globals['_ENDPOINTREQUESTMESSAGE']._serialized_end=106
  _globals['_ENDPOINTRESPONSEMESSAGE']._serialized_start=108
  _globals['_ENDPOINTRESPONSEMESSAGE']._serialized_end=204
  _globals['_ENDPOINTREQUEST']._serialized_start=207
  _globals['_ENDPOINTREQUEST']._serialized_end=599
  _globals['_ENDPOINTREQUEST_METADATAENTRY']._serialized_start=413
  _globals['_ENDPOINTREQUEST_METADATAENTRY']._serialized_end=460
  _globals['_ENDPOINTREQUEST_HEADERSENTRY']._serialized_start=462
  _globals['_ENDPOINTREQUEST_HEADERSENTRY']._serialized_end=528
  _globals['_ENDPOINTREQUEST_PARAMETERSENTRY']._serialized_start=530
  _globals['_ENDPOINTREQUEST_PARAMETERSENTRY']._serialized_end=599
  _globals['_ENDPOINTRESPONSE']._serialized_start=602
  _globals['_ENDPOINTRESPONSE']._serialized_end=999
  _globals['_ENDPOINTRESPONSE_METADATAENTRY']._serialized_start=413
  _globals['_ENDPOINTRESPONSE_METADATAENTRY']._serialized_end=460
  _globals['_ENDPOINTRESPONSE_HEADERSENTRY']._serialized_start=462
  _globals['_ENDPOINTRESPONSE_HEADERSENTRY']._serialized_end=528
  _globals['_ENDPOINTRESPONSE_PARAMETERSENTRY']._serialized_start=530
  _globals['_ENDPOINTRESPONSE_PARAMETERSENTRY']._serialized_end=599
  _globals['_STRINGLIST']._serialized_start=1001
  _globals['_STRINGLIST']._serialized_end=1029
  _globals['_REGISTERREQUEST']._serialized_start=1031
  _globals['_REGISTERREQUEST']._serialized_end=1088
  _globals['_REGISTERRESPONSE']._serialized_start=1090
  _globals['_REGISTERRESPONSE']._serialized_end=1151
  _globals['_PLUGINENDPOINT']._serialized_start=1154
  _globals['_PLUGINENDPOINT']._serialized_end=1422
  _globals['_LOGGINGARGS']._serialized_start=1424
  _globals['_LOGGINGARGS']._serialized_end=1437
  _globals['_LOGFIELD']._serialized_start=1439
  _globals['_LOGFIELD']._serialized_end=1477
  _globals['_LOGMESSAGE']._serialized_start=1479
  _globals['_LOGMESSAGE']._serialized_end=1575
  _globals['_PLUGINSERVICE']._serialized_start=1646
  _globals['_PLUGINSERVICE']._serialized_end=1857
# @@protoc_insertion_point(module_scope)
